@@ -42,6 +42,8 @@
 
 #include <math.h>
 
+#include "guppi-math.h"
+
 enum {
   ARG_0
 };
@@ -420,6 +422,7 @@ guppi_axis_update (GnomeCanvasItem *item, double *update_affine, ArtSVP *clip_pa
               ++n_texts;
             }
       
+          // Urgh, this is not efficient but the numbers are small
           while (n_texts > gint(ticks.size()))
             {
               GnomeCanvasItem* txt = (GnomeCanvasItem*)ga->texts->data;
