@@ -38,6 +38,23 @@
 #include "vfont.h"
 #include "util.h"
 
+#ifndef M_PI
+#define	M_PI		3.14159265358979323846
+#endif
+
+#ifdef GNOME_ENABLE_DEBUG
+#include <time.h>
+static inline void start_time(guint* store)
+{
+  *store = clock();
+}
+static inline void check_time(guint* store)
+{
+  guint now = clock();
+  g_debug("%u clocks elapsed", now - *store);
+}
+#endif
+
 void 
 PlotUtil::round_numbers (int goal,
                          double* min, double* max,
@@ -178,12 +195,91 @@ PlotUtil::position_string(PositionType pos)
 
 /////////////////////// Sorts
 
+template <class SType>
+static inline void
+psort_swap(SType* left, SType* right)
+{
+  SType tmp = *left;
+  *left = *right;
+  *right = tmp;
+}
+
+// The shell sort isn't a total lost cause if the array has lots of 
+//  duplicate values; so the qsort bails out to this if necessary.
+template <class XType, class YType>
+static inline void
+parallel_shell_sort(XType* xsorted, YType* ysorted, int N)
+{
+  // h is our increment
+
+  int h = 1;
+  if (N < 14)
+    h = 1;
+  else 
+    {
+      while (h < N) 
+        h = 3*h + 1;
+      
+      h /= 3;
+      h /= 3;
+    }
+
+  while (h > 0) 
+    {
+      // Insertion sort
+
+      int i = h;
+      while (i < N)
+        {
+          XType x_pivot_value = xsorted[i];
+          YType y_pivot_value = ysorted[i];
+
+          g_assert(i >= h);
+
+          int j = i - h;
+
+          while (j >= 0 && xsorted[j] > x_pivot_value)
+            {
+              g_assert(j+h < N);
+              g_assert(j+h >= 0);
+              xsorted[j+h] = xsorted[j];
+              ysorted[j+h] = ysorted[j];
+
+              j -= h;
+            }
+          xsorted[j+h] = x_pivot_value;
+          ysorted[j+h] = y_pivot_value;
+          
+          ++i;
+        }
+
+      h /= 3;
+    }
+
+#ifdef GNOME_ENABLE_DEBUG_SLOW
+  if (N > 1)
+    {
+      guint q = 1;
+      XType last_x = xsorted[0];
+      while (q < N)
+        {
+          if (last_x > xsorted[q])
+            g_warning("Shell sort: %u out of order!", q);
+
+          last_x = xsorted[q];
+          ++q;
+        }
+    }
+#endif
+}
+
 template <class XType, class YType>
 static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
 {
   if (N == 1) return;
 
-  unsigned int* stck = new unsigned int[N+7]; // too large but definitely big enough.
+  // FIXME figure out the max stack size for real.
+  unsigned int* stck = new unsigned int[N+7]; // far too large but definitely big enough.
   unsigned int top = 2;
   
   // The stack contains pairs of indices denoting ranges to sort
@@ -195,84 +291,197 @@ static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
       unsigned int strt = stck[top-2];
       unsigned int stop = stck[top-1];
 
-      unsigned int diff = stop - strt;
-
-      if (diff > 2)
+      unsigned int diff = stop - strt;     
+      
+      if (diff > 3)
         {
-          unsigned int pivot = strt + diff/2;
+          // qsort, baby
+          const unsigned int pivot = strt + diff/2;
+          const unsigned int last = stop - 1; // remember that stop is one past
+      
+          // Rearrange hi, lo, and pivot values to be in order; this 
+          //  gives us a better pivot and lets us instantly reduce the 
+          //  partition size. Idea from glibc
+      
+          if (xsorted[pivot] < xsorted[strt])
+            {
+              psort_swap(&xsorted[strt], &xsorted[pivot]);
+              psort_swap(&ysorted[strt], &ysorted[pivot]);
+            }
+      
+          if (xsorted[last] < xsorted[pivot])
+            {
+              psort_swap(&xsorted[last], &xsorted[pivot]);
+              psort_swap(&ysorted[last], &ysorted[pivot]);
+          
+              if (xsorted[pivot] < xsorted[strt])
+                {
+                  psort_swap(&xsorted[strt], &xsorted[pivot]);
+                  psort_swap(&ysorted[strt], &ysorted[pivot]);
+                }
+            }
+
+          //          g_assert(xsorted[strt] <= xsorted[pivot]);
+          // g_assert(xsorted[pivot] <= xsorted[last]);
+
+          // The above ensures the last value is >= pivot so we don't
+          // check it again.
 
           // Move pivot value to front out of the way
-          XType tmp = xsorted[strt];
-          xsorted[strt] = xsorted[pivot];
-          xsorted[pivot] = tmp;
-
-          YType tmp2 = ysorted[strt];
-          ysorted[strt] = ysorted[pivot];
-          ysorted[pivot] = tmp2;
           
-          // Partitition around pivot value
+          psort_swap(&xsorted[strt], &xsorted[pivot]);
+          psort_swap(&ysorted[strt], &ysorted[pivot]);
+          
+          // Partition around pivot value
+          const XType pivot_value = xsorted[strt];
           unsigned int lo = strt + 1;
-          unsigned int hi = stop;
+          unsigned int hi = stop - 1; // We know the last value is >= pivot
+          
+          // Simple swap partitioning - note that the glibc way 
+          //  is much faster. 
           
           while (lo < hi)
             {
-              if (xsorted[lo] < xsorted[strt]) // strt is now pivot value
+              while (lo < hi && xsorted[lo] < pivot_value)
                 ++lo;
-              else if (xsorted[--hi] < xsorted[strt])
+              
+              // Now go the other way
+              
+              while (lo < hi && xsorted[--hi] >= pivot_value)
+                ; // empty loop
+              
+              // Either lo == hi or 
+              // xsorted[lo] >= pivot_value && xsorted[hi] < pivot_value
+              
+#ifdef GNOME_ENABLE_DEBUG_SLOW
+              g_assert((lo == hi) || 
+                       (xsorted[lo] >= pivot_value && xsorted[hi] < pivot_value));
+#endif
+              
+              if (lo < hi)
                 {
-                  tmp = xsorted[lo];
-                  xsorted[lo] = xsorted[hi];
-                  xsorted[hi] = tmp;
-                  tmp2 = ysorted[lo];
-                  ysorted[lo] = ysorted[hi];
-                  ysorted[hi] = tmp2;
-                }
-            }
-          
+                  psort_swap(&xsorted[lo], &xsorted[hi]);
+                  psort_swap(&ysorted[lo], &ysorted[hi]);
+                  ++lo;
+                  // remember that hi remains one past the one to check...
+                }           
+            }          
+
           // put pivot back
           --lo; // lo is at the pivot point
-          tmp = xsorted[lo];
-          xsorted[lo] = xsorted[strt];
-          xsorted[strt] = tmp;
-          tmp2 = ysorted[lo];
-          ysorted[lo] = ysorted[strt];
-          ysorted[strt] = tmp2;
-
+          
           // lo is the new pivot point; it won't be sorted again
+
+          // In the worst case (bad pivot), we move forward only 1;
+          //  we'll switch to shell sort in that case, since most likely
+          //  we have a bunch of duplicate points - also, the stored pivot
+          //  value is already in the right spot, so we don't need to 
+          //  put it back.
           
           top -= 2; // pop strt/stp
 
-          if (strt < lo)
+          if (strt == lo)
             {
+          
+#ifdef GNOME_ENABLE_DEBUG_SLOW
+              // Check our invariant
+              {
+                guint i = strt;
+                while (i < stop)
+                  {
+                    if (i < lo)
+                      g_assert(xsorted[i] < xsorted[lo]);
+                    else if (i > lo)
+                      g_assert(xsorted[i] >= xsorted[lo]);
+                
+                    ++i;
+                  }
+              }
+#endif
+
+              // We know strt == lo so there's only the second partition left to sort
+              parallel_shell_sort(&xsorted[lo+1], &ysorted[lo+1], stop-lo-1);
+            }
+          else 
+            {
+              // Put pivot back; not necessary in little_progress case
+              psort_swap(&xsorted[lo], &xsorted[strt]);
+              psort_swap(&ysorted[lo], &ysorted[strt]);
+
+          
+#ifdef GNOME_ENABLE_DEBUG_SLOW
+              // Check our invariant
+              {
+                guint i = strt;
+                while (i < stop)
+                  {
+                    if (i < lo)
+                      g_assert(xsorted[i] < xsorted[lo]);
+                    else if (i > lo)
+                      g_assert(xsorted[i] >= xsorted[lo]);
+                
+                    ++i;
+                  }
+              }
+#endif
+
+
+              // if (strt < lo) // already known, since we have progress.
+
               stck[top] = strt;
               stck[top+1] = lo; // pivot is the limit
               top += 2;
+              
+              if (lo < stop)
+                {
+                  stck[top] = lo + 1; // one past pivot
+                  stck[top+1] = stop;
+                  top += 2;
+                }
             }
-          
-          if (lo < stop)
+        }
+      else if (diff == 3)
+        {
+          const unsigned int pivot = strt + 1;
+
+          --stop;
+
+          if (xsorted[pivot] < xsorted[strt])
             {
-              stck[top] = lo + 1; // one past pivot
-              stck[top+1] = stop;
-              top += 2;
+              psort_swap(&xsorted[strt], &xsorted[pivot]);
+              psort_swap(&ysorted[strt], &ysorted[pivot]);
             }
+      
+          if (xsorted[stop] < xsorted[pivot])
+            {
+              psort_swap(&xsorted[stop], &xsorted[pivot]);
+              psort_swap(&ysorted[stop], &ysorted[pivot]);
+          
+              if (xsorted[pivot] < xsorted[strt])
+                {
+                  psort_swap(&xsorted[strt], &xsorted[pivot]);
+                  psort_swap(&ysorted[strt], &ysorted[pivot]);
+                }
+            }
+
+          //          g_assert(xsorted[strt] <= xsorted[pivot]);
+          //          g_assert(xsorted[pivot] <= xsorted[stop]);
+
+          top -= 2;
         }
       else if (diff == 2)
         {
           --stop;
           if (xsorted[strt] > xsorted[stop])
             {
-              XType tmp = xsorted[strt];
-              xsorted[strt] = xsorted[stop];
-              xsorted[stop] = tmp;
-              YType tmp2 = ysorted[strt];
-              ysorted[strt] = ysorted[stop];
-              ysorted[stop] = tmp2;
+              psort_swap(&xsorted[strt], &xsorted[stop]);
+              psort_swap(&ysorted[strt], &ysorted[stop]);
             }
           top -= 2;
         }
       else 
         {
-          // range of 0 or 1 - just pop
+          // range of 0 or 1 - just pop (well, shouldn't have bothered to push...)
           top -= 2;
         }
     }
@@ -280,6 +489,8 @@ static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
   delete [] stck;
 
 #ifdef GNOME_ENABLE_DEBUG
+  //  check_time(&timer);
+
   if (N > 1)
     {
       guint q = 1;
@@ -287,12 +498,14 @@ static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
       while (q < N)
         {
           if (last_x > xsorted[q])
-            g_warning("Out of order!");
+            g_warning("%u out of order!", q);
 
           last_x = xsorted[q];
           ++q;
         }
     }
+  //  check_time(&timer);
+
 #endif
 }
 
