@@ -450,7 +450,8 @@ SortedPair::clear()
 void 
 SortedPair::recreate()
 {
-  clear();
+  if (tiles_ != 0)
+    g_warning("Recreating tiles we already have!");
 
   if (otherdata_ == 0 || keydata_ == 0) 
     return;
