@@ -24,11 +24,6 @@
  * USA
  */
 
-// There is plenty of cut-and-paste within this file; 
-// I would be grateful to anyone who abstracted it nicely (hint hint)
-// Two good starts: a XyAction base class, and function 
-// to "get the nearest point"
-
 #include "xyactions.h"
 #include "xyplot.h"
 #include "ezrubberband.h"
@@ -93,6 +88,135 @@ get_points_in_region(XyDataLayer* layer,
     }
 }
 
+static bool
+get_nearest_point(XyPlot* xyp,
+                  double x, double y,
+                  XyDataLayer** best_layer,
+                  guint* best_point)
+{
+  *best_layer = 0;
+  *best_point = 0;
+
+  // Collect all scatter/line plots that we want to affect.
+  vector<XyDataLayer*> layers;
+      
+  XyPlotState::iterator i   = xyp->state()->begin();
+  XyPlotState::iterator end = xyp->state()->end();
+      
+  while (i != end)
+    {
+      if (!(*i)->hidden() && !(*i)->masked() &&
+          (*i)->type_registry()->is_a((*i)->type(), XyLayer::DataLayer))
+        {
+          XyDataLayer* layer = (XyDataLayer*)(*i)->cast_to_type(XyLayer::DataLayer);
+          
+          g_assert(layer != 0);
+          
+          layers.push_back(layer);
+        }
+
+      ++i;
+    }
+
+  // Reverse the plots, because we want to prefer the one 
+  //  on top.
+  reverse(layers.begin(), layers.end());
+
+  bool found_points = false;
+  double best_so_far = 0.0;
+
+  vector<XyDataLayer*>::iterator j = layers.begin();
+
+  while (j != layers.end())
+    {
+      XyDataLayer* layer = *j;
+
+      TypeRegistry* tr = layer->type_registry();
+
+      g_assert(tr != 0);
+
+      PlotUtil::PositionType xaxis = layer->x_axis();
+      PlotUtil::PositionType yaxis = layer->y_axis();
+      const Transform & xtrans = xyp->state()->trans(xaxis);
+      const Transform & ytrans = xyp->state()->trans(yaxis);
+
+      double vx1, vy1, vx2, vy2;
+
+      double closeness;
+      
+      if (tr->is_a(layer->type(), XyLayer::ScatterLayer))
+        closeness = 
+          ((XyScatter*)layer->cast_to_type(XyLayer::ScatterLayer))->largest_style();
+      else
+        closeness = 1.0; // for "corners" of a line plot
+
+      vx1 = xtrans.inverse(x - closeness);
+      vy1 = ytrans.inverse(y + closeness);
+      vx2 = xtrans.inverse(x + closeness);
+      vy2 = ytrans.inverse(y - closeness);
+
+      vector<guint> found;
+
+      get_points_in_region(layer,
+                           vx1, vy1, vx2, vy2, 
+                           found);      
+
+      if (!found.empty())
+        {
+          ScalarData* xdat = layer->x_data();
+          ScalarData* ydat = layer->y_data();
+
+          g_assert(xdat && ydat);
+
+          vector<double> xs;
+          vector<double> ys;
+
+          vector<guint>::iterator fi = found.begin();           
+          while (fi != found.end())
+            {
+              xs.push_back(xdat->get_scalar(*fi));
+              ys.push_back(ydat->get_scalar(*fi));
+                               
+              ++fi;
+            }
+
+          g_assert(xs.size() == ys.size());
+
+          // Find best match
+          guint pi = 0;
+          double best = closeness*10;
+          guint best_index = 0; // best index into found; found contains real index
+          while (pi < xs.size())
+            {
+              // hypot() can take negative arguments, I'm declaring.
+              double dist = PlotUtil::hypot((xs[pi] - x), 
+                                            (ys[pi] - y));
+
+              if (dist <= best)
+                {
+                  best = dist;
+                  best_index = pi;
+                }
+
+              ++pi;
+            }
+
+          if (!found_points || (best < best_so_far))
+            {
+              best_so_far = best;
+              *best_layer = layer;
+              *best_point = found[best_index];
+            }
+
+          found_points = true;
+        }
+
+      ++j;
+    }
+
+  return found_points;
+}
+
 
 // Select a rectangular area and resize axes to match it
 // (i.e., zoom in on a rectangular area.)
@@ -223,8 +347,15 @@ AxisShrinker::snapped(double x1, double y1,
   while (i < PlotUtil::PositionTypeEnd)
     {
       const PlotUtil::PositionType pos = (PlotUtil::PositionType) i;      
-      const Transform & trans = sp->state()->trans(pos);
 
+      // Don't affect hidden axes
+      if (sp->state()->axis(pos).hidden())
+        {
+          ++i;
+          continue;
+        }
+
+      const Transform & trans = sp->state()->trans(pos);
       double v1, v2;
 
       // Since the transform contains a flip, we need to map 'y2' which is
@@ -250,7 +381,13 @@ AxisShrinker::snapped(double x1, double y1,
     }
 
   // Move to new area.
-  sp->state()->push_axis_bounds(positions, starts, stops);
+  if (!positions.empty())
+    sp->state()->push_axis_bounds(positions, starts, stops);
+  else
+    {
+      sp->set_status(_("All axes are hidden."));
+      gdk_beep();
+    }
 
   ungrab(sp);
 }
@@ -324,7 +461,39 @@ AxisExpander::event(Plot* p, GdkEvent* event)
   return FALSE;
 }
 
-#if 0
+/// Panning
+
+// pan proportional to distance of mouse pointer from the middle 
+//  of the plot
+static void 
+pan_axis(XyPlot* xyp, PlotUtil::PositionType pos, double item_coord)
+{
+  const Transform & trans = xyp->state()->trans(pos);
+
+  double v = trans.inverse(item_coord);
+
+  double old1 = xyp->state()->axis(pos).start();
+  double old2 = xyp->state()->axis(pos).stop();
+
+  double middle = old1 + (old2 - old1)/2;
+
+  double delta = v - middle;
+
+  double new1 = old1 + delta;
+  double new2 = old2 + delta;
+
+  Axis* a = xyp->state()->checkout_axis(pos);  
+  
+  if (a != 0)
+    {
+      a->set_start(new1);
+      a->set_stop(new2);
+      
+      xyp->state()->checkin_axis(pos, a);
+    }
+  else 
+    g_warning("Couldn't check out axis to modify it!");
+}
 
 /// Horizontal Panning 
 
@@ -363,36 +532,15 @@ HPanner::event(Plot* p, GdkEvent* event)
 
   if (event->type == GDK_BUTTON_PRESS)
     {
-      const Transform & xtrans = sp->state()->x_trans();
-  
+      const Transform & strans = sp->state()->trans(PlotUtil::SOUTH);
+
       double x = event->button.x;
       double dummy = 0.0;
 
       gnome_canvas_item_w2i(sp->item(), &x, &dummy);
-      
-      x = xtrans.inverse(x);
 
-      double x1 = sp->state()->x_axis().start();
-      double x2 = sp->state()->x_axis().stop();
-
-      double middle = x1 + (x2-x1)/2;
-
-      double delta = x - middle;
-
-      double xstart = x1 + delta;
-      double xstop = x2 + delta;
-
-      Axis* a = sp->state()->checkout_x_axis();  
-
-      if (a != 0)
-        {
-          a->set_start(xstart);
-          a->set_stop(xstop);
-
-          sp->state()->checkin_x_axis(a);
-        }
-      else 
-        g_warning("Couldn't check out X axis to modify it!");
+      pan_axis(sp, PlotUtil::NORTH, x);
+      pan_axis(sp, PlotUtil::SOUTH, x);
      
       return TRUE;
     }
@@ -438,43 +586,21 @@ VPanner::event(Plot* p, GdkEvent* event)
 
   if (event->type == GDK_BUTTON_PRESS)
     {
-      const Transform & ytrans = sp->state()->y_trans();
-  
       double y = event->button.y;
       double dummy = 0.0;
 
       gnome_canvas_item_w2i(sp->item(), &dummy, &y);
-      
-      y = ytrans.inverse(y);
 
-      double y1 = sp->state()->y_axis().start();
-      double y2 = sp->state()->y_axis().stop();
-
-      double middle = y1 + (y2-y1)/2;
-
-      double delta = y - middle;
-
-      double ystart = y1 + delta;
-      double ystop = y2 + delta;
-
-      Axis* a = sp->state()->checkout_y_axis();  
-
-      if (a != 0)
-        {
-          a->set_start(ystart);
-          a->set_stop(ystop);
-
-          sp->state()->checkin_y_axis(a);
-        }
-      else 
-        g_warning("Couldn't check out Y axis to modify it!");
-     
+      pan_axis(sp, PlotUtil::WEST, y);
+      pan_axis(sp, PlotUtil::EAST, y);
+           
       return TRUE;
     }
 
   return FALSE;
 }
 
+
 // Zoom in to a clicked point action
 
 class Telescope : public Action {
@@ -516,37 +642,67 @@ Telescope::event(Plot* p, GdkEvent* event)
 
   if (event->type == GDK_BUTTON_PRESS)
     {
-      const Transform & xtrans = sp->state()->x_trans();
-      const Transform & ytrans = sp->state()->y_trans();
-  
       double x = event->button.x;
       double y = event->button.y;
 
       gnome_canvas_item_w2i(sp->item(), &x, &y);
+
+      vector<PlotUtil::PositionType> positions;
+      vector<double> starts;
+      vector<double> stops;
       
-      x = xtrans.inverse(x);
-      y = ytrans.inverse(y);
+      int i = 0;
+      while (i < PlotUtil::PositionTypeEnd)
+        {
+          const PlotUtil::PositionType pos = (PlotUtil::PositionType) i;      
+          
+          // Don't affect hidden axes
+          if (sp->state()->axis(pos).hidden())
+            {
+              ++i;
+              continue;
+            }
+          
+          const Transform & trans = sp->state()->trans(pos);
+          double v;
+          
+          double v1 = sp->state()->axis(pos).start();
+          double v2 = sp->state()->axis(pos).stop();
+                    
+          if (PlotUtil::vertical(pos))
+            {
+              v = trans.inverse(y);
+            }
+          else 
+            {
+              v = trans.inverse(x);
+            }
 
-      double x1 = sp->state()->x_axis().start();
-      double x2 = sp->state()->x_axis().stop();
-      double y1 = sp->state()->y_axis().start();
-      double y2 = sp->state()->y_axis().stop();
+          double newv1 = v - (v - v1) * percent_;
+          double newv2 = v + (v2 - v) * percent_;
 
-      double xstart = x - (x - x1) * percent_;
-      double xstop = x + (x2 - x) * percent_;
-      double ystart = y - (y - y1) * percent_;
-      double ystop = y + (y2 - y) * percent_;
-
-      sp->state()->push_axis_bounds(xstart, ystart, xstop, ystop);
+          positions.push_back(pos);
+          starts.push_back(newv1);
+          stops.push_back(newv2);
+          
+          ++i;
+        }
       
+      // Move to new area.
+      if (!positions.empty())
+        sp->state()->push_axis_bounds(positions, starts, stops);
+      else
+        {
+          sp->set_status(_("All axes are hidden."));
+          gdk_beep();
+        }
+ 
       return TRUE;
     }
 
   return FALSE;
 }
 
-#endif
-
 // Brush action, applies a style to points around the mouse pointer.
 
 class Brush : public Action {
@@ -975,10 +1131,66 @@ PointID::do_update(XyPlot* sp)
     }
 }
 
-#if 0
-
 /////////////////// Recenter, RecenterX, RecenterY
 
+static void
+recenter_axes(XyPlot* xyp, 
+              double x, double y,
+              const PlotUtil::PositionType axes[], 
+              guint naxes)
+{
+  vector<PlotUtil::PositionType> positions;
+  vector<double> starts;
+  vector<double> stops;
+
+  int i = 0;
+  while (i < (int)naxes)
+    {
+      PlotUtil::PositionType pos = axes[i];
+
+      if (xyp->state()->axis(pos).hidden())
+        {
+          ++i;
+          continue;
+        }
+
+      const Transform & trans = xyp->state()->trans(pos);      
+
+      double v1 = xyp->state()->axis(pos).start();
+      double v2 = xyp->state()->axis(pos).stop();
+
+      double v;
+
+      if (PlotUtil::vertical(pos))
+        {
+          v = trans.inverse(y);
+        }
+      else 
+        {
+          v = trans.inverse(x);
+        }
+
+      double len = v2 - v1;
+
+      double new_v1 = v - len/2;
+      double new_v2 = v + len/2;
+      
+      positions.push_back(pos);
+      starts.push_back(new_v1);
+      stops.push_back(new_v2);
+
+      ++i;
+    }
+  
+  if (!positions.empty())
+    xyp->state()->push_axis_bounds(positions, starts, stops);
+  else
+    {
+      xyp->set_status(_("All axes hidden!"));
+      gdk_beep();
+    }
+}
+
 class Recenter : public Action {
 public:
   Recenter() 
@@ -1021,36 +1233,15 @@ Recenter::event(Plot* p, GdkEvent* event)
       y = event->button.y;
 
       gnome_canvas_item_w2i(sp->item(), &x, &y);
-      
-      const Transform & xtrans = sp->state()->x_trans();
-      const Transform & ytrans = sp->state()->y_trans();
-  
-      double vx, vy;
 
-      // Since the transform contains a flip, we need to map 'y2' which is
-      // toward the bottom of the screen to 'vy1' which is also toward the
-      // bottom of the screen.
+      static const PlotUtil::PositionType axes[] = {
+        PlotUtil::NORTH,
+        PlotUtil::SOUTH,
+        PlotUtil::EAST,
+        PlotUtil::WEST
+      };
 
-      vx = xtrans.inverse(x);
-      vy = ytrans.inverse(y);
-
-      const double x1 = sp->state()->x_axis().start();
-      const double x2 = sp->state()->x_axis().stop();
-      const double y1 = sp->state()->y_axis().start();
-      const double y2 = sp->state()->y_axis().stop();
-
-      // Should "just work" if x1 > x2 or y1 > y2, keeping things arranged 
-      // that way after recenter
-
-      const double width = x2-x1;
-      const double height = y2-y1;
-
-      const double xstart = vx - width/2;
-      const double xstop = vx + width/2;
-      const double ystart = vy - height/2;
-      const double ystop = vy + height/2;
-
-      sp->state()->push_axis_bounds(xstart, ystart, xstop, ystop);
+      recenter_axes(sp, x, y, axes, sizeof(axes)/sizeof(axes[0]));
 
       return TRUE;
     }
@@ -1101,28 +1292,13 @@ RecenterY::event(Plot* p, GdkEvent* event)
       y = event->button.y;
 
       gnome_canvas_item_w2i(sp->item(), &x, &y);
-      
-      const Transform & ytrans = sp->state()->y_trans();
-  
-      double vy;
 
-      // Since the transform contains a flip, we need to map 'y2' which is
-      // toward the bottom of the screen to 'vy1' which is also toward the
-      // bottom of the screen.
+      static const PlotUtil::PositionType axes[] = {
+        PlotUtil::EAST,
+        PlotUtil::WEST
+      };
 
-      vy = ytrans.inverse(y);
-
-      const double x1 = sp->state()->x_axis().start();
-      const double x2 = sp->state()->x_axis().stop();
-      const double y1 = sp->state()->y_axis().start();
-      const double y2 = sp->state()->y_axis().stop();
-
-      const double height = y2-y1;
-
-      const double ystart = vy - height/2;
-      const double ystop = vy + height/2;
-
-      sp->state()->push_axis_bounds(x1, ystart, x2, ystop);
+      recenter_axes(sp, x, y, axes, sizeof(axes)/sizeof(axes[0]));
 
       return TRUE;
     }
@@ -1173,30 +1349,13 @@ RecenterX::event(Plot* p, GdkEvent* event)
       y = event->button.y;
 
       gnome_canvas_item_w2i(sp->item(), &x, &y);
-      
-      const Transform & xtrans = sp->state()->x_trans();
-  
-      double vx;
 
-      // Since the transform contains a flip, we need to map 'y2' which is
-      // toward the bottom of the screen to 'vy1' which is also toward the
-      // bottom of the screen.
+      static const PlotUtil::PositionType axes[] = {
+        PlotUtil::NORTH,
+        PlotUtil::SOUTH
+      };
 
-      vx = xtrans.inverse(x);
-
-      const double x1 = sp->state()->x_axis().start();
-      const double x2 = sp->state()->x_axis().stop();
-
-      // Should "just work" if x1 > x2 or y1 > y2, keeping things arranged 
-      // that way after recenter
-
-      const double width = x2-x1;
-
-      const double xstart = vx - width/2;
-      const double xstop = vx + width/2;
-
-      sp->state()->push_axis_bounds(xstart, sp->state()->y_axis().start(),
-                                    xstop, sp->state()->y_axis().stop());
+      recenter_axes(sp, x, y, axes, sizeof(axes)/sizeof(axes[0]));
 
       return TRUE;
     }
@@ -1204,8 +1363,6 @@ RecenterX::event(Plot* p, GdkEvent* event)
   return FALSE;
 }
 
-#endif
-
 ////////////////// Select-and-style
 
 
@@ -1403,7 +1560,6 @@ Styler::cancelled()
   current_ = 0;
 }
 
-#if 0
 
 //////////////// PointDrag
 // Shows nearby points in statusbar
@@ -1417,6 +1573,7 @@ public:
       tooltip_(_("Change value of a point by dragging it")),
       in_drag_(false),
       drag_point_(0),
+      drag_layer_(0),
       fixedtip_(0)
     {}
   virtual ~PointDrag() {}
@@ -1435,8 +1592,11 @@ private:
 
   bool in_drag_;
   guint drag_point_;
+  XyDataLayer* drag_layer_;
 
   GtkWidget* fixedtip_;
+
+  void end_drag(XyPlot* xyp, guint32 when);
 };
 
 gint
@@ -1460,85 +1620,31 @@ PointDrag::event(Plot* p, GdkEvent* event)
                              &wx, &wy, &state);
       gnome_canvas_window_to_world(sp->item()->canvas, wx, wy, &x, &y);
       gnome_canvas_item_w2i(sp->item(), &x, &y);
-      
-      const Transform & xtrans = sp->state()->x_trans();
-      const Transform & ytrans = sp->state()->y_trans();
-  
-      double vx1, vy1, vx2, vy2;
 
-      // Since the transform contains a flip, we need to map 'y2' which is
-      // toward the bottom of the screen to 'vy1' which is also toward the
-      // bottom of the screen.
-      const double closeness = sp->state()->largest_style();
-      vx1 = xtrans.inverse(x - closeness);
-      vy1 = ytrans.inverse(y + closeness);
-      vx2 = xtrans.inverse(x + closeness);
-      vy2 = ytrans.inverse(y - closeness);
+      guint best_point;
+      XyDataLayer* best_layer;
 
-      vector<guint> found;
-
-      get_points_in_region(sp->state(),
-                           vx1, vy1, vx2, vy2,
-                           found);
-         
-      if (found.empty())
+      if (get_nearest_point(sp, x, y, &best_layer, &best_point))
         {
-          // The snprintf is pointless now, I know.
-          gchar buf[128];
-          g_snprintf(buf, 127, _("No points nearby."));
+          g_assert(best_layer != 0);
+          
+          ScalarData* xdat = best_layer->x_data();
+          ScalarData* ydat = best_layer->y_data();
 
-          sp->set_status(buf);
-          gdk_beep();
-        }
-      else 
-        {
-          ScalarData* xdat = sp->state()->x_data();
-          ScalarData* ydat = sp->state()->y_data();
-
-          g_assert(xdat && ydat);
-
-          vector<double> xs;
-          vector<double> ys;
-
-          vector<guint>::iterator fi = found.begin();           
-          while (fi != found.end())
-            {
-              xs.push_back(xdat->get_scalar(*fi));
-              ys.push_back(ydat->get_scalar(*fi));
-                               
-              ++fi;
-            }
-
-          g_assert(xs.size() == ys.size());
-
-          // Find best match
-          guint pi = 0;
-          double best = closeness*10;
-          guint best_index = 0; // best index into found; found contains real index
-          while (pi < xs.size())
-            {
-              // hypot() can take negative arguments, I'm declaring.
-              double dist = PlotUtil::hypot((xs[pi] - x), 
-                                            (ys[pi] - y));
-
-              if (dist <= best)
-                {
-                  best = dist;
-                  best_index = pi;
-                }
-
-              ++pi;
-            }
+          g_assert(xdat && ydat); // because there were points
 
           char buf[128];
-          g_snprintf(buf, 128, _("Dragging point %u: (%g, %g)"),
-                     found[best_index],
-                     xs[best_index], ys[best_index]);
+          g_snprintf(buf, 128, _("Dragging point %u: (%g, %g) Layer %s"),
+                     best_point,
+                     xdat->get_scalar(best_point), 
+                     ydat->get_scalar(best_point),
+                     best_layer->name().c_str());
 
-          drag_point_ = found[best_index];
+          drag_point_ = best_point;
+          drag_layer_ = best_layer;
               
           GdkCursor* ptr = gdk_cursor_new(GDK_HAND2);
-
+          
           gnome_canvas_item_grab(sp->item(),
                                  GDK_POINTER_MOTION_MASK | 
                                  GDK_BUTTON_RELEASE_MASK | 
@@ -1558,16 +1664,26 @@ PointDrag::event(Plot* p, GdkEvent* event)
 
           fixedtip_ = gtk_fixedtip_new();
 
-          g_snprintf(buf, 128, _("Point: %u\nX: %g\nY: %g"),
-                     found[best_index],
-                     xs[best_index], ys[best_index]);
+          g_snprintf(buf, 128, _("Layer: %s\nPoint: %u\nX: %g\nY: %g"),
+                     drag_layer_->name().c_str(),
+                     best_point,
+                     xdat->get_scalar(best_point), 
+                     ydat->get_scalar(best_point));
 
           gtk_fixedtip_set_text(GTK_FIXEDTIP(fixedtip_), buf);
 
           GdkWindow* canvas_window = GTK_WIDGET(sp->item()->canvas)->window;
 
           gtk_fixedtip_popup(GTK_FIXEDTIP(fixedtip_), -50, -50, canvas_window);
-
+          
+        }
+      else
+        {
+          gchar buf[128];
+          g_snprintf(buf, 127, _("No points nearby."));
+          
+          sp->set_status(buf);
+          gdk_beep();
         }
 
       return TRUE;
@@ -1576,14 +1692,7 @@ PointDrag::event(Plot* p, GdkEvent* event)
     {
       if (event->type == GDK_BUTTON_RELEASE)
         {
-          in_drag_ = false;
-
-          gtk_widget_destroy(fixedtip_);
-
-          gnome_canvas_item_ungrab(sp->item(), event->button.time);
-
-          ungrab(p);
-
+          end_drag(sp, event->button.time);
           return TRUE;
         }
       else if (event->type == GDK_MOTION_NOTIFY)
@@ -1597,8 +1706,8 @@ PointDrag::event(Plot* p, GdkEvent* event)
           
           gnome_canvas_item_w2i(sp->item(), &x, &y);
 
-          const Transform & xtrans = sp->state()->x_trans();
-          const Transform & ytrans = sp->state()->y_trans();
+          const Transform & xtrans = sp->state()->trans(drag_layer_->x_axis());
+          const Transform & ytrans = sp->state()->trans(drag_layer_->y_axis());
   
           double vx, vy;
           
@@ -1611,16 +1720,26 @@ PointDrag::event(Plot* p, GdkEvent* event)
           // vx and vy are our new values for the point, but we have to make 
           // sure the point hasn't disappeared...
 
-          if (drag_point_ < sp->state()->npoints())
+          if (drag_point_ < drag_layer_->npoints())
             {
-              sp->state()->x_data()->set_scalar(drag_point_, vx);
-              sp->state()->y_data()->set_scalar(drag_point_, vy);
+              ScalarData* xdat = drag_layer_->x_data();
+              ScalarData* ydat = drag_layer_->y_data();
+
+              // Because we have a grab, we assume no one has removed the data
+              g_assert(xdat && ydat);
+
+              xdat->set_scalar(drag_point_, vx);
+              ydat->set_scalar(drag_point_, vy);
+
               char buf[128];
-              g_snprintf(buf, 128, _("Point %u (%g, %g)"), drag_point_, vx, vy);
+              g_snprintf(buf, 128, _("Point %u (%g, %g) from Layer %s"), 
+                         drag_point_, vx, vy,
+                         drag_layer_->name().c_str());
               sp->set_status(string(buf));
 
               // Also show status in the fixed tip
-              g_snprintf(buf, 128, _("Point: %u\nX: %g\nY: %g"),
+              g_snprintf(buf, 128, _("Layer: %s\nPoint: %u\nX: %g\nY: %g"),
+                         drag_layer_->name().c_str(),
                          drag_point_, 
                          vx, vy);
 
@@ -1634,10 +1753,7 @@ PointDrag::event(Plot* p, GdkEvent* event)
             {
               sp->set_status(_("Point no longer exists!"));
               gdk_beep();
-              gtk_widget_destroy(fixedtip_);
-              gnome_canvas_item_ungrab(sp->item(), event->button.time);
-              ungrab(p);
-              in_drag_ = false;
+              end_drag(sp, event->button.time);
             }
           
           return TRUE;
@@ -1646,7 +1762,21 @@ PointDrag::event(Plot* p, GdkEvent* event)
 
   return FALSE;
 }
-#endif
+
+void
+PointDrag::end_drag(XyPlot* xyp, guint32 when)
+{
+  in_drag_ = false;
+  
+  drag_point_ = 0;
+  drag_layer_ = 0;
+
+  gtk_widget_destroy(fixedtip_);
+  
+  gnome_canvas_item_ungrab(xyp->item(), when);
+  
+  ungrab(xyp);
+}
 
 /////////////////////////// Menu entries
 
@@ -1841,16 +1971,13 @@ guppi_gnome_init_xy()
   ps->add_action(XyPlot::ID, new Brush);
   ps->add_action(XyPlot::ID, new PointID);
   ps->add_action(XyPlot::ID, new Styler);
-
-#if 0
   ps->add_action(XyPlot::ID, new HPanner);
   ps->add_action(XyPlot::ID, new VPanner);
   ps->add_action(XyPlot::ID, new Telescope);
+  ps->add_action(XyPlot::ID, new PointDrag);
   ps->add_action(XyPlot::ID, new Recenter);
   ps->add_action(XyPlot::ID, new RecenterX);
   ps->add_action(XyPlot::ID, new RecenterY);
-  ps->add_action(XyPlot::ID, new PointDrag);
-#endif
 
   ps->add_menuentry(XyPlot::ID, new XyPrint);
   ps->add_menuentry(XyPlot::ID, new XyProperties);
