@@ -904,24 +904,33 @@ XyPlot::change_axis(XyPlotState* state,
 
   if (axis_labels_[pos] != 0)
     {
-      GFont* gfont = static_cast<GFont*>(axis.label_font());
-      // Get 1.0 scale font, since GnomeCanvasText does scaling for us.
-      GdkFont* gdkfont = gfont ? gfont->gdk_font(1.0) : 0;
-
-      if (gdkfont != 0)
+      if (axis.hidden())
         {
-          // keep in sync with else block
-          gnome_canvas_item_set(axis_labels_[pos],
-                                "text", axis.label().c_str(),
-                                "font_gdk", gdkfont,
-                                NULL);
+          gnome_canvas_item_hide(axis_labels_[pos]);
         }
-      else
+      else 
         {
-          gnome_canvas_item_set(axis_labels_[pos],
-                                "text", axis.label().c_str(),
-                                "font", "fixed", // kinda bad - breaks our font cache.
-                                NULL);
+          GFont* gfont = static_cast<GFont*>(axis.label_font());
+          // Get 1.0 scale font, since GnomeCanvasText does scaling for us.
+          GdkFont* gdkfont = gfont ? gfont->gdk_font(1.0) : 0;
+          
+          if (gdkfont != 0)
+            {
+              // keep in sync with else block
+              gnome_canvas_item_set(axis_labels_[pos],
+                                    "text", axis.label().c_str(),
+                                    "font_gdk", gdkfont,
+                                    NULL);
+            }
+          else
+            {
+              gnome_canvas_item_set(axis_labels_[pos],
+                                    "text", axis.label().c_str(),
+                                    "font", "fixed", // kinda bad - breaks our font cache.
+                                    NULL);
+            }
+
+          gnome_canvas_item_show(axis_labels_[pos]);
         }
     }  
 }
@@ -933,13 +942,21 @@ XyPlot::change_size(XyPlotState* state, double width, double height)
 
   if (bg_ != 0)
     {
-      gnome_canvas_item_set(bg_,
-                            "x2", width,
-                            NULL);
-      
-      gnome_canvas_item_set(bg_,
-                            "y2", height,
-                            NULL);  
+      // this is pretty much just paranoia; GnomeCanvasRE doesn't like
+      // 0 dimensions
+      if (width <= 0.0 || height <= 0.0)
+        gnome_canvas_item_hide(bg_);
+      else 
+        {
+          gnome_canvas_item_set(bg_,
+                                "x2", width,
+                                NULL);
+          
+          gnome_canvas_item_set(bg_,
+                                "y2", height,
+                                NULL);
+          gnome_canvas_item_show(bg_);
+        }
     }
 
   gint i = 0;
